@@ -6,18 +6,27 @@ from django.contrib.auth import REDIRECT_FIELD_NAME, authenticate, login
 from django.core.urlresolvers import reverse
 from django.http import HttpResponse, HttpResponseRedirect
 from django.shortcuts import render
-from openid.consumer.consumer import Consumer, SUCCESS, CANCEL, FAILURE
+from openid.consumer.consumer import Consumer, SUCCESS, CANCEL, FAILURE, SETUP_NEEDED
 from openid.consumer.discover import DiscoveryFailure
 from openid.extensions import sreg
 
 from .store import DjangoOpenIDStore
 
+
 def get_sreg_settings():
     return getattr(settings, 'SREG_OPTIONAL', ['email', 'fullname'])
 
+
 def login_begin(request, redirect_field_name=REDIRECT_FIELD_NAME,
-                template_name='openid/response.html'):
-    redirect_to = request.REQUEST.get(redirect_field_name, '')
+                template_name='openid/response.html', immediate=False):
+
+    # Set redirect from settings file for immediate redirects.
+    # We must set this here for checkid_immediate since the request
+    # will be coming from the provider.
+    if immediate:
+        redirect_to = settings.LOGIN_REDIRECT_URL
+    else:
+        redirect_to = request.REQUEST.get(redirect_field_name, '')
 
     openid_url = settings.OPENID_SSO_SERVER_URL
 
@@ -42,7 +51,7 @@ def login_begin(request, redirect_field_name=REDIRECT_FIELD_NAME,
         return_to += '&' if '?' in return_to else '?'
         return_to += urllib.urlencode({redirect_field_name: redirect_to})
 
-    return render_openid_request(request, openid_request, return_to)
+    return render_openid_request(request, openid_request, return_to, immediate)
 
 
 def login_complete(request, redirect_field_name=REDIRECT_FIELD_NAME,
@@ -80,6 +89,15 @@ def login_complete(request, redirect_field_name=REDIRECT_FIELD_NAME,
         return HttpResponseRedirect(
             redirect_to=sanitise_redirect_url(redirect_to)
         )
+
+    if openid_response.status == SETUP_NEEDED:
+        # setup_needed means the immediate request failed, and we need to redirect the user to
+        # either a server side login page (openid_response.setup_url) or a consumer side login page.
+        setup_url_param = urllib.urlencode({'setup_url': openid_response.setup_url})
+        return HttpResponseRedirect(
+            "%s?%s" % (reverse('login_setup_needed'), setup_url_param)
+        )
+
     message = 'Unknown OpenID response type: %r' % openid_response.status
     return render(
         request, template_name, {'message': message}, status=400
@@ -117,17 +135,17 @@ def make_consumer(request):
     return Consumer(session, store)
 
 
-def render_openid_request(request, openid_request, return_to, trust_root=None):
+def render_openid_request(request, openid_request, return_to, immediate=False, trust_root=None):
     """Render an OpenID authentication request."""
     if trust_root is None:
         trust_root = request.build_absolute_uri('/')
 
     if openid_request.shouldSendRedirect():
-        redirect_url = openid_request.redirectURL(trust_root, return_to)
+        redirect_url = openid_request.redirectURL(trust_root, return_to, immediate)
         return HttpResponseRedirect(redirect_url)
     else:
         form_html = openid_request.htmlMarkup(
-            trust_root, return_to, form_tag_attrs={'id': 'openid_message'}
+            trust_root, return_to, immediate, form_tag_attrs={'id': 'openid_message'}
         )
         return HttpResponse(form_html, content_type='text/html;charset=UTF-8')
 
@@ -137,3 +155,11 @@ def parse_openid_response(request):
     current_url = request.build_absolute_uri()
     consumer = make_consumer(request)
     return consumer.complete(dict(request.REQUEST.items()), current_url)
+
+
+def setup_needed(request, template_name="openid/setup_needed.html"):
+    """
+    Render a page in response to openid server returning setup_needed response
+    """
+    setup_url = request.GET.get('setup_url', '')
+    return render(request, template_name, {'setup_url': setup_url})
